//
//  DeviceIDManager.m
//  AppPiza
//
//  Copyright (c) 2014 CJSC "Computing Forces"
//

#import "DeviceIDManager.h"

NSString * const kDeviceUUIDKey = @"deviceUUID";




@interface DeviceIDManager ()

@property (nonatomic, readwrite, retain) NSString *currentDeviceUUID;

@end




@implementation DeviceIDManager

#pragma mark -
#pragma mark Singleton implementation
static DeviceIDManager *_sharedInstance = nil;



+ (instancetype)sharedInstance {
    if (!_sharedInstance)
        _sharedInstance = [[DeviceIDManager alloc] init];
    
    return _sharedInstance;
}


- (id)init {
    self = [super init];
    if (self) {
        // Load UUID from settings
        NSUserDefaults *ud = [NSUserDefaults standardUserDefaults];
        self.currentDeviceUUID = [ud objectForKey:kDeviceUUIDKey];
        
        // Generate new UUID
        if (!self.currentDeviceUUID) {
            self.currentDeviceUUID = [DeviceIDManager GenerateUUID];
            
            // Store new UUID to settings
            [ud setObject:self.currentDeviceUUID forKey:kDeviceUUIDKey];
        }
        
        [ud synchronize];
    }
    return self;
}


- (void)dealloc {
    [_currentDeviceUUID release];
    
    [super dealloc];
}


- (oneway void)release {
}


- (id)retain {
    return self;
}


- (id)autorelease {
    return self;
}


- (NSUInteger)retainCount {
    return INT_MAX;
}




#pragma mark -
#pragma mark Generating UUID
+ (NSString *)GenerateUUID {
    CFUUIDRef theUUID = CFUUIDCreate(NULL);
    CFStringRef string = CFUUIDCreateString(NULL, theUUID);
    CFRelease(theUUID);
    return (__bridge NSString *)string;
}

@end
