//
//  ActivityVC.m
//  AppPiza
//
//  Copyright (c) 2014 CJSC "Computing Forces"
//

#import "ActivityVC.h"


@interface ActivityVC ()

@property (retain, nonatomic) IBOutlet UIImageView *ivActivity;
@property (retain, nonatomic) IBOutlet UILabel *lblMessage;

@end



@implementation ActivityVC


#pragma mark -
#pragma mark Singleton implementation
static ActivityVC *_sharedInstance = nil;



+ (instancetype)sharedInstance {
    if (!_sharedInstance)
        _sharedInstance = [[[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"ActivityVC"] retain];
    
    return _sharedInstance;
}



- (void)dealloc {
    [_ivActivity release];
    [_lblMessage release];
    
    [super dealloc];
}


- (oneway void)release {
}


- (id)retain {
    return self;
}


- (id)autorelease {
    return self;
}


- (NSUInteger)retainCount {
    return INT_MAX;
}




#pragma mark -
- (void)viewDidLoad {
    [super viewDidLoad];
	
    self.ivActivity.animationImages = [self animationFramesWithFormat:@"refresh-animation-%d.png" framesCount:9];
    self.ivActivity.animationDuration = 0.5;
}



- (NSArray *)animationFramesWithFormat:(NSString *)format framesCount:(NSUInteger)framesCount {
    NSMutableArray *frames = [NSMutableArray arrayWithCapacity:framesCount];
    
    for (NSUInteger i = 1; i <= framesCount; i++) {
        NSString *frameName = [NSString stringWithFormat:format, i];
        UIImage *frame = [UIImage imageNamed:frameName];
        if (frame) [frames addObject:frame];
    }
    
    return frames;
}



- (void)showInViewController:(UIViewController *)parentVC {
    if (![parentVC isViewLoaded]) return;
    
    [self hide];
    
    self.view.frame = parentVC.view.bounds;
    [parentVC.view addSubview:self.view];
    
    [parentVC addChildViewController:self];
    [self didMoveToParentViewController:parentVC];
    
    [self.ivActivity startAnimating];
}



- (void)hide {
    if (!self.parentViewController) return;
    
    [self.view removeFromSuperview];
    
    [self removeFromParentViewController];
    [self didMoveToParentViewController:nil];
    
    [self.ivActivity stopAnimating];
}

@end
