//
//  CartVC.m
//  AppPiza
//
//  Copyright (c) 2014 CJSC "Computing Forces"
//

#import "CartVC.h"

#import "ServiceRequestBuilder.h"
#import "CartManager.h"
#import "DeviceIDManager.h"
#import "ServerSettingsManager.h"

#import "CartCell.h"

#import "InitiateOrderVC.h"
#import "ActivityVC.h"



@interface CartVC () <UITableViewDelegate, UITableViewDataSource>

@property (nonatomic, readwrite, retain) IBOutlet UITableView *tableView;
@property (retain, nonatomic) IBOutlet UILabel *lblAmount;

@property (nonatomic, readwrite, retain) NSArray *cartItems;

@end



@implementation CartVC {
    NSUInteger _currentOrderId;
    float _currentAmount;
}



- (id)initWithCoder:(NSCoder *)aDecoder {
    self = [super initWithCoder:aDecoder];
    if (self) {
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(notificationCartChanged:) name:kCartChangedNotification object:nil];
    }
    return self;
}



- (void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self name:kCartChangedNotification object:nil];
    
    [_cartItems release];
    
    [_tableView release];
    [_lblAmount release];
    
    [super dealloc];
}



- (void)viewDidLoad {
    [super viewDidLoad];
    
    [self updateAmount];
}



- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    
    [self updateItems];
}



- (void)updateItems {
    self.cartItems = [[CartManager sharedInstance] getAllItems];
    
    [self.tableView reloadData];
}



#pragma mark - Table view data source
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return [self.cartItems count];
}



- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    CartCell *cell = [tableView dequeueReusableCellWithIdentifier:@"CartCell" forIndexPath:indexPath];
    
    CartItem *cartItem = self.cartItems[indexPath.row];
    cell.cartItem = cartItem;
    
    cell.removeButton.tag = indexPath.row;
    
    if (!cell.initialized) {
        [cell.removeButton addTarget:self action:@selector(actionRemove:) forControlEvents:UIControlEventTouchUpInside];
    }
    
    
    for (id obj in cell.subviews) {
        if ([obj isKindOfClass:[UIScrollView class]]) {
            UIScrollView *scroll = (UIScrollView *) obj;
            scroll.delaysContentTouches = NO;
            break;
        }
    }
    
    
    cell.initialized = YES;
    return cell;
}



- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 84.0f;
}



#pragma mark -
#pragma mark Actions
- (void)actionRemove:(UIButton *)sender {
    CartItem *cartItem = self.cartItems[sender.tag];
    [[CartManager sharedInstance] removeAllGoods:cartItem.goods];
    
    [self updateItems];
}



- (IBAction)actionPay:(id)sender {
    if (![self.cartItems count]) {
        [self showErrorMessage:@"Корзина пуста!"];
        return;
    }
    
    [[ActivityVC sharedInstance] showInViewController:self.tabBarController];
    
    NSString *cartString = [[CartManager sharedInstance] packedCartString];
    NSString *deviceId = [DeviceIDManager sharedInstance].currentDeviceUUID;
    ServiceRequest *createRequest = [[ServiceRequestBuilder sharedInstance] requestCreateOrder:cartString deviceId:deviceId];
    
    createRequest.successBlock = ^ (ServiceRequest *request) {
        [[ActivityVC sharedInstance] hide];
        
        ServiceResponseJSON *response = (id)request.response;
        
        _currentOrderId = [response.response[@"order_id"] integerValue];
        _currentAmount = [response.response[@"amount"] floatValue];
        
        [self performSegueWithIdentifier:@"sgShowInitiateOrder" sender:self];
    };
    
    createRequest.failBlock = ^ (ServiceRequest *request) {
        [[ActivityVC sharedInstance] hide];
        
        [self showErrorMessage:request.error.localizedDescription];
    };
    
    [[ServiceManager sharedInstance] processRequest:createRequest];
}



- (void)showErrorMessage:(NSString *)message {
    [[[[UIAlertView alloc] initWithTitle:@"Ошибка оплаты" message:message delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil] autorelease] show];
}



#pragma mark -
#pragma mark Navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    if ([segue.identifier isEqualToString:@"sgShowInitiateOrder"]) {
        InitiateOrderVC *initiateOrderVC = segue.destinationViewController;
        initiateOrderVC.orderId = _currentOrderId;
        initiateOrderVC.amount = _currentAmount;
    }
}



#pragma mark -
#pragma mark Notifications
- (void)notificationCartChanged:(NSNotification *)notification {
    [self updateAmount];
}



- (void)updateAmount {
    NSUInteger totalCount = 0;
    float amount = 0.0f;
    for (CartItem *cartItem in [[CartManager sharedInstance] getAllItems]) {
        totalCount += cartItem.count;
        amount += cartItem.goods.price * cartItem.count;
    }
    
    self.navigationController.tabBarItem.badgeValue = totalCount ? [@(totalCount) stringValue] : nil;
    self.lblAmount.text = [[ServerSettingsManager sharedInstance] stringForPrice:amount];
}

@end
