<?php

require_once("api_globals.php");
require_once("../database.php");

$db = ConnectDB();



// --- Creating transaction
function CreateTransaction($cart, $deviceId) {
    if (count($cart) == 0) {
        return BuildResponseArray(kApiResponseCodeBadParameters, kApiResponseMessageBadParameters, null);
    }
    
    // Get good information
    $price = CalculateCartPriceAndFetchGoods($cart);
    
    if ($price === false) {
        return BuildResponseArray(kApiResponseCodeGoodNotFound, kApiResponseMessageGoodNotFound, null);
    }
    
    // Add new order
    $orderId = CreateOrder($cart, $price, $deviceId);
    
    // Build response array
    $responseData = array(
       'order_id' => $orderId,
       'amount' => $price
    );
    return BuildResponseArray(kApiResponseCodeOk, kApiResponseMessageOk, $responseData);
}



// --- Initiating transaction
function InitiateTransaction($orderId, $phone, $address) {
    global $settings;
    
    if ($phone == null) {
        return BuildResponseArray(kApiResponseCodeBadParameters, kApiResponseMessageBadParameters, null);
    }
    
    // Get order information
    $query = "select * from orders where id = $orderId";
    $result = mysql_query($query);
    $order = mysql_fetch_array($result);
    mysql_freeresult($result);
    
    if ($order == null) {
        return BuildResponseArray(kApiResponseCodeOrderNotFound, kApiResponseMessageOrderNotFound, null);
    }
    
    $invoiceId = (int)$order['wminvoiceid'];
    if ($invoiceId != null) {
        return BuildResponseArray(kApiResponseCodeTransactionAlreadyInitiated, kApiResponseMessageTransactionAlreadyInitiated, null);
    }
    
    $amount = (float)$order['amount'];
    
    // Send transaction request
    $requestData = array(
        'wmid' => $settings['payeeWmid'],
        'lmi_payee_purse' => $settings['payeePurse'],
        'lmi_payment_no' => (int)$orderId,
        'lmi_payment_amount' => $amount,
        'lmi_payment_desc' => $settings['description'],
        
        'lmi_clientnumber' => $phone,
        'lmi_clientnumber_type' => kClientNumberTypeMobilePhone,
        'lmi_sms_type' => (int)$settings['confirmationType'],
        
        'lang' => $settings['language'],
        'emulated_flag' => (int)$settings['emulated']
    );
    
    // Signing request
    if ($settings['signMethid'] == kSignMethodSecretKey) {
        $requestData['secret_key'] = $settings['secretKey'];
    }
    else if ($settings['signMethid'] == kSignMethodMD5) {
        // wmid + lmi_payee_purse + lmi_payment_no + lmi_clientnumber + lmi_clientnumber_type + secret_key
        $signingString = $requestData['wmid'] . $requestData['lmi_payee_purse'] . $requestData['lmi_payment_no'] . $requestData['lmi_clientnumber'] . $requestData['lmi_clientnumber_type'] . $requestData['secret_key'];
        $requestData['md5'] = md5($signingString);
    }

    $jsonResponse = SendJsonRequest("https://merchant.webmoney.ru/conf/xml/XMLTransRequest.asp", $requestData);
    if ($jsonResponse == false) {
        UpdateOrder($orderId, kOrderStateFailed, $phone, $address);
        return BuildResponseArray(kApiResponseCodeUnknownError, kApiResponseMessageUnknownError, null);
    }
    else {
        if ($jsonResponse['retval'] == kX20ResponseCodeSuccess) {
            $operationInfo = $jsonResponse['operation'];
            UpdateOrder($orderId, kOrderStateInitiated, $phone, $address, $operationInfo['wminvoiceid'], $operationInfo['wmtransid'], $operationInfo['realsmstype']);
        }
        else {
            UpdateOrder($orderId, kOrderStateFailed, $phone, $address);
            return BuildResponseArray(kApiResponseCodeUnknownError, $jsonResponse['userdesc'], null);
        }
    }
    
    // Build response array
    return BuildResponseArray(kApiResponseCodeOk, $jsonResponse['userdesc'], null);
}



// --- Confirming transaction
function ConfirmTransaction($orderId, $code) {
    global $settings;
    
    if ($orderId == null || $code == null) {
        return BuildResponseArray(kApiResponseCodeBadParameters, kApiResponseMessageBadParameters, null);
    }
    
    // Get order information
    $query = "select * from orders where id = $orderId";
    $result = mysql_query($query);
    $order = mysql_fetch_array($result);
    mysql_freeresult($result);
    
    if ($order == null) {
        return BuildResponseArray(kApiResponseCodeOrderNotFound, kApiResponseMessageOrderNotFound, null);
    }
    
    $invoiceId = (int)$order['wminvoiceid'];
    if ($invoiceId == null) {
        return BuildResponseArray(kApiResponseCodeTransactionNotInitiated, kApiResponseMessageTransactionNotInitiated, null);
    }
    
    // Send transaction confirmation
    $requestData = array(
        'wmid' => $settings['payeeWmid'],
        'lmi_payee_purse' => $settings['payeePurse'],
        'lmi_wminvoiceid' => $invoiceId,
        'lmi_clientnumber_code' => $code,
        
        'lang' => $settings['language']
    );
    
    // Signing request
    if ($settings['signMethid'] == kSignMethodSecretKey) {
        $requestData['secret_key'] = $settings['secretKey'];
        $requestData['md5'] = '';
        $requestData['sign'] = '';
    }
    else if ($settings['signMethid'] == kSignMethodMD5) {
        // wmid + lmi_payee_purse + lmi_wminvoiceid + lmi_clientnumber_code
        $signingString = $requestData['wmid'] . $requestData['lmi_payee_purse'] . $requestData['lmi_wminvoiceid'] . $requestData['lmi_clientnumber_code'];
        $requestData['md5'] = md5($signingString);
        $requestData['secret_key'] = '';
        $requestData['sign'] = '';
    }
    
    $jsonResponse = SendJsonRequest("https://merchant.webmoney.ru/conf/xml/XMLTransConfirm.asp", $requestData);
    if ($jsonResponse == false) {
        UpdateOrder($orderId, kOrderStateFailed);
        return BuildResponseArray(kApiResponseCodeUnknownError, kApiResponseMessageUnknownError, null);
    }
    else {
        $operationInfo = $jsonResponse['operation'];
        $retval = $jsonResponse['retval'];
        
        if ($retval == kX20ResponseCodeSuccess) {
            UpdateOrder($orderId, kOrderStateConfirmed, null, null, null, null, null, OffsetServerFormatTime($operationInfo['operdate'], kServerTimeOffset), $operationInfo['purpose'], $operationInfo['pursefrom'], $operationInfo['wmidfrom']);
        }
        else if ($retval == kX20ResponseCodeCancelled) {
            UpdateOrder($orderId, kOrderStateCancelled, null, null, null, null, null, OffsetServerFormatTime($operationInfo['operdate'], kServerTimeOffset), $operationInfo['purpose'], $operationInfo['pursefrom'], $operationInfo['wmidfrom']);
        }
        else if ($retval == kX20ResponseWrongCode || $retval == kX20ResponseWrongCode2) {
            $failsCount = $order['confirm_fails'];
            IncreaseOrderConfirmFailsCount($orderId);
            
            if ($failsCount + 1 >= kMaxConfirmFailsCount) {
                UpdateOrder($orderId, kOrderStateFailed);
                return BuildResponseArray(kApiResponseCodeTooManyConfirmationFails, kApiResponseMessageTooManyConfirmationFails, null);
            }
            else {
                return BuildResponseArray(kApiResponseCodeWrongConfirmationCode, kApiResponseMessageWrongConfirmationCode, null);
            }
        }
        else {
            UpdateOrder($orderId, kOrderStateFailed);
            return BuildResponseArray(kApiResponseCodeUnknownError, $jsonResponse['userdesc'], null);
        }
    }
    
    // Build response array
    $responseData = array(
       'order_id' => $orderId
    );
    return BuildResponseArray(kApiResponseCodeOk, $jsonResponse['userdesc'], $responseData);
}



// --- Get orders list
function GetOrdersList($deviceId) {
    if ($deviceId == null) {
        return BuildResponseArray(kApiResponseCodeBadParameters, kApiResponseMessageBadParameters, null);
    }
    
    // Fetch data from DB
    $deviceIdEscaped = mysql_escape_string($deviceId);
    $query = "select * from orders where device_id=\"$deviceIdEscaped\"";
    
    $result = mysql_query($query);
    $ordersList = array();
    while ($line = mysql_fetch_array($result)) {
        $order = array(
            'id' => $line['id'],
            'phone' => $line['phone'],
            'state' => $line['state'],
            'amount' => $line['amount'],
            'date' => $line['date'],
            'address' => $line['address']
        );
        
        array_push($ordersList, $order);
    }
        
    mysql_free_result($result);
    
    // Build response array
    return BuildResponseArray(kApiResponseCodeOk, kApiResponseMessageOk, $ordersList);
}



// --- Helpers
function CalculateCartPriceAndFetchGoods($cart) {
    // Build good ids list
    $goodIdsStr = GoodIdsStringForCart($cart);
    
    // Fetch goods
    $query = "select * from goods where id in ($goodIdsStr)";
    $result = mysql_query($query);
    
    $price = 0;
    $countFound = 0;
    while($good = mysql_fetch_array($result)) {
        $goodId = (int)$good['id'];
        $cart[$goodId]['good'] = $good;
        
        $price += (float)$good['price'] * $cart[$goodId]['count'];
        $countFound++;
    }
    
    mysql_freeresult($result);
    
    return ($countFound == count($cart)) ? $price : false;
}



function GoodIdsStringForCart($cart) {
    $goodIds = array_keys($cart);
    
    $goodIdsStr = implode(', ', $goodIds);
    return $goodIdsStr;
}



function OffsetServerFormatTime($serverFormatTime, $offset) {
    $parsed = date_parse_from_format(kServerTimeFormat, $serverFormatTime);
    
    $timestamp = mktime(
        $parsed['hour'], 
        $parsed['minute'], 
        $parsed['second'], 
        $parsed['month'], 
        $parsed['day'], 
        $parsed['year']
    );
        
    $timestamp += $offset;
        
    return date(kServerTimeFormat, $timestamp);
}



function CreateOrder($cart, $amount, $deviceId) {
    // Insert order
    $state = kOrderStateAdded;
    $deviceIdEscaped = mysql_escape_string($deviceId);
    $currentDate = gmdate(kServerTimeFormat);
    
    $query = "insert into orders (state, amount, device_id, date) values ($state, $amount, \"$deviceIdEscaped\", \"$currentDate\")";
    
    $result = mysql_query($query);
    if ($result === false) return 0;
    
    $orderId = mysql_insert_id();
    
    // Insert cart items
    foreach ($cart as $goodId => $cartItem) {
        $count = $cartItem['count'];
        $query = "insert into orders_to_goods (order_id, good_id, count) values ($orderId, $goodId, $count)";
        
        $result = mysql_query($query);
        if ($result === false) return 0;
    }
    
    return $orderId;
}



function UpdateOrder($orderId, $state, $phone = null, $address = null, $wminvoiceid = null, $wmtransid = null, $realsmstype = null, $date = null, $purpose = null, $wmidFrom = null, $purseFrom = null) {
    $phoneEscaped = ($phone != null) ? mysql_escape_string($phone) : null;
    $phoneAddress = ($address != null) ? mysql_escape_string($address) : null;
    
    $query = "update orders set state = $state";
    if ($phoneEscaped !== null) $query .= ", phone = \"$phoneEscaped\"";
    if ($phoneAddress !== null) $query .= ", address = \"$phoneAddress\"";
    if ($wminvoiceid !== null) $query .= ", wminvoiceid = $wminvoiceid";
    if ($wmtransid !== null) $query .= ", wmtransid = $wmtransid";
    if ($realsmstype !== null) $query .= ", realsmstype = $realsmstype";
    if ($date !== null) $query .= ", date = \"$date\"";
    if ($purpose !== null) $query .= ", purpose = \"$purpose\"";
    if ($wmidFrom !== null) $query .= ", wmid_from = \"$wmidFrom\"";
    if ($purseFrom !== null) $query .= ", purse_from = \"$purseFrom\"";
    $query .= " where id = $orderId";
    
    //echo($query);
    
    $result = mysql_query($query);
    return $result;
}



function IncreaseOrderConfirmFailsCount($orderId) {
    $query = "update orders set confirm_fails = confirm_fails + 1 where id = $orderId";
    $result = mysql_query($query);
    return $result;
}



function ParseCartString($cartString) {
    $parts = explode(',', $cartString);
    
    $cart = array();
    
    foreach($parts as $goodString) {
        $goodParts = explode('-', $goodString);
        $goodId = $goodParts[0];
        $count = $goodParts[1];
        
        if ($goodId == 0) continue;
        
        $cart[$goodId] = array (
            'count' => ($count == null) ? 1 : $count
        );
    }
    
    return $cart;
}







// --- Getting request parameters
$action = $_GET['action'];

$responseData = null;
if ($action == 'create') {
    $cartString = $_GET['cart'];
    $cart = ParseCartString($cartString);
    $deviceId = $_GET['device_id'];
    
    $responseData = CreateTransaction($cart, $deviceId);
}
else if ($action == 'initiate') {
    $orderId = (int)$_GET['order_id'];
    $phone = $_GET['phone'];
    $address = $_GET['address'];

    $responseData = InitiateTransaction($orderId, $phone, $address);
}
else if ($action == 'confirm') {
    $orderId = (int)$_GET['order_id'];
    $code = $_GET['code'];
    
    $responseData = ConfirmTransaction($orderId, $code);
}
else if ($action == 'list') {
    $deviceId = $_GET['device_id'];
    
    $responseData = GetOrdersList($deviceId);
}


// --- Building response
header('Content-Type: text/json');

$responseJson = json_encode($responseData);
echo($responseJson);

DisconnectDB($db);

?>